#!/usr/bin/env python3
# Time-stamp: "2021-10-17 15:08:30 (ylab)"

from . import dsp, general, ml, plt, resource, stats

__copyright__ = "Copyright (C) 2021 Yusuke Watanabe"
__version__ = "0.0.29"
__license__ = "GPL3.0"
__author__ = "ywatanabe1989"
__author_email__ = "ywata1989@gmail.com"
__url__ = "https://github.com/ywatanabe1989/mngs"
